function [B S t NC xxdata state]=mox(state,fcn_eval,IR,nv,Pz,fcn_param,varargin)
% Author: Fayyaz ul Amir Afsar Minhas (fayyazafsar <at> gmail <dot> com)
% Affiliation: Dept. of Computer and Information Sciences, Pakistan
% Institute of Engineering and Applied Sciences (PIEAS), PO Nilore,
% Islamabad, Pakistan.
% Web: fayyazafsar.googlepages.com
% Date: 1145HRS26JAN2011
% Implementation of "MOX: A Novel Global Optimization Algorithm Inspired
% from  from Oviposition Site Selection and Egg Hatching Inhibition in
% Mosquitoes", Submitted for review in Journal of Applied Soft Computing
% PARAMETER DETAILS:
% INPUT:
% fcn_eval: The function to be minimized (handle or a string name)
% IR: Range of exploration, i.e. [-MIN to +MAX] for input variables
% nv: number of variables (dimensions)
% Pz: Initial population
% fcn_param: parameters to the function (if you want to pass any!) else [].
% If some parameters are to be passed to the function then the function
% must accept two input arguments (first one would be the input vector (x)
% and the other can be a structure of parameters)
% optional parameters: (specified as paramter,value pairs, default shown in
% square [] brackets
%   'dd': The state of MOX is displayed after every 'dd' generations [0]
%   'FMIN': The goal value [1e-3]
%   'MAX_NC': The maximun number of function evaluations [1e6]
%   'MAX_ADULTS': Adult Population size [70]
%   'MAX_EGGS': Max. population of eggs [14]
%   'N_EGGS': Max. Number of eggs created by each impregnation [4]
%   'PM': Probability of mating [0.5]
%   'EGG_P': Probability of laying an egg [1.0]
%   'PR': prob. of each component of male to be changed in mating [0.5]
%   'PC': prob. of each component of male to be changed in random shift [0.1]
%   'gg': Gamma (greedy) parameter [0.95]
%   '
% See the paper for more details about parameters
% OUTPUT:
%   B: best solution
%   S: Success or failure (1/0) indicator for convergence
%   t: Number of iterations for convergence
%   NC: Number of function evalutations required for convergence
%   xxdata: data for plotting convergence graphs
% EXAMPLE: Running for the Rastrigin's function
% [B S t NC xxdata state]=mox([],'rastriginsfcn',[-5.12 5.12],30,[],[],'dd',10,'FMIN',1e-3,'MAX_ADULTS',80);

%% PARSING STARTS
% names={'dd','fcn_eval','IR','nv','FMIN','MAX_NC','MAX_ADULTS','MAX_EGGS','N_EGGS','PC','EGG_P','PR','PM','gg','fcn_param'};
% defaults={0,'rastriginsfcn',[-5.12 5.12],30,1e-3,1e6,70,14,4,0.5,1,0.5,0.1,0.95,[]};

withstate=1;
if(isempty(state))
    withstate=0;
    state=struct;
    p = inputParser;
    p.addParamValue('dd', 0, @(x) isnumeric(x) );
    p.addParamValue('FMIN', 1e-3, @(x) isnumeric(x));
    p.addParamValue('MAX_NC', 1e6, @(x) isnumeric(x) && x>0);
    p.addParamValue('MAX_ADULTS', 70, @(x) isnumeric(x) && x>0);
    p.addParamValue('MAX_EGGS', 14, @(x) isnumeric(x) && x>=0);
    p.addParamValue('N_EGGS', 4, @(x) isnumeric(x) && x>=0);
    p.addParamValue('PM', 0.5, @(x) isnumeric(x) && x>=0 && x<=1);
    p.addParamValue('EGG_P', 1, @(x) isnumeric(x) && x>=0 && x<=1);
    p.addParamValue('PR', 0.5, @(x) isnumeric(x) && x>=0 && x<=1);
    p.addParamValue('PC', 0.1, @(x) isnumeric(x) && x>=0 && x<=1);
    p.addParamValue('gg', 0.95, @(x) isnumeric(x) && x>=0 && x<=1);
    p.addParamValue('MAX_T', inf, @(x) isnumeric(x) && x>=0);
    p.parse(varargin{:});
    p=p.Results;
    state.params=p;
    state.fcn_eval=fcn_eval;
    state.IR=IR;
    state.nv=nv;
    state.Pz=Pz;
    state.fcn_param=fcn_param;
    rand('state',sum(100*clock))
    
else
    p=state.params;
    fcn_eval=state.fcn_eval;
    fcn_param=state.fcn_param;
    IR=state.IR;
    %     state,fcn_eval,IR,nv,Pz,fcn_param,varargin
    nv=state.nv;
    Pz=state.Pz;
    G=state.G;
    haseggs=state.haseggs;
    FF=state.FF;
    EF=state.EF;
    vv=state.vv;
    B=state.B;
    indxB=state.indxB;
    P=state.P;
    E=state.E;
    BETA=state.BETA;
    Bs=state.Bs;
    Sx=state.Sx;
    bsindx=state.bsindx;
    xxdata=state.xxdata;
    NC=state.NC;
    
end
dd=round(p.dd);FMIN=p.FMIN;MAX_NC=round(p.MAX_NC);MAX_ADULTS=round(p.MAX_ADULTS);
MAX_EGGS=round(p.MAX_EGGS);N_EGGS=round(p.N_EGGS);PC=p.PM;EGG_P=p.EGG_P;PR=p.PR;
PM=p.PC;gg=p.gg;MAX_T=p.MAX_T;
 H=30;
 MAX_BETA=1;%diff(IR)/3;
   
    MIN_BETA=0.4;
if(~withstate)
    % if not resuming an old run
    %% create initial population of adults (gender is selected randomly)
    INITIAL_MAX_ADULTS=MAX_ADULTS;
    
    BETA0=1;
    Bs=zeros(1,H);Sx=Bs;
    Bs(1)=[BETA0];bsindx=2;Sx(1)=1;
    BETA=BETA0;
    Rv=repmat(IR,nv,1);
    [~,spz]=size(Pz);spz=min(spz,INITIAL_MAX_ADULTS);
    P=repmat(Rv(:,1),1,INITIAL_MAX_ADULTS)+(rand(nv,INITIAL_MAX_ADULTS)).*repmat(Rv(:,2)-Rv(:,1),1,INITIAL_MAX_ADULTS);%location data for each mosquito6
    P(:,1:spz)=Pz;
    haseggs=zeros(1,INITIAL_MAX_ADULTS); %contains the number eggs for a female
    G=rand(1,INITIAL_MAX_ADULTS)>0.5; %gender, 1= female, 0=male
    %% initially there are no eggs
    E=[];
    %% perform these step for a number of time steps
    vv=inf;
    NC=0;
    FF=zeros(1,size(P,2));
    for p=1:size(P,2)
        [FF(p) ]=evalfcn(fcn_eval,P(:,p),NC,fcn_param);
    end
    NC=NC+size(P,2);
    EF=[];
    
    dd=round(dd);
end
t=0;
while(1)
    t=t+1;
    if((dd>0)&&(~mod(t,dd)))
        disp(['Iter:' num2str(t-1) ' bestf:' num2str(min(FF)) ' NC:' num2str(NC) ' Beta: ' num2str(BETA) ' f %:' num2str(100*nnz(G)/length(G))])
    end
    sp=size(P,2);
    %% augment the current population with eggs to evaluate fitness
    P=[P E];
    %     FF=zeros(1,size(P,2));
    FF=[FF EF];
    
    % replace the worst performing adults with better performing
    % individuals from eggs
    [FF indx]=sort(FF,'ascend');
    indx=indx(1:min(length(indx),MAX_ADULTS));
    FF=FF(1:min(length(indx),MAX_ADULTS));
    P=P(:,indx);
    G2=zeros(size(indx));H2=zeros(size(indx));
    
    % For original adults keep their eggs and gender and for newly borns
    % determine gender randomly and initialize their eggs to zero
    for k=1:size(P,2)
        if(indx(k)<=sp)
            G2(k)=G(indx(k));
            H2(k)=haseggs(indx(k));
        else
            G2(k)=rand>0.5;
            H2(k)=0;
        end
    end
    haseggs=H2;
    G=G2;
    % remove the newly borns from the eggs list
    eggs_to_adults=find(indx>sp);
    E(:,indx(eggs_to_adults)-sp)=[];
    EF(indx(eggs_to_adults)-sp)=[];
    findx=find(G);
    mindx=find(~G);
    % save elite
    [vv indxB]=min(FF);B=P(:,indxB);
    xxdata(t,1)=NC;xxdata(t,2)=vv;
    
    if(bsindx>H)
        div=H;
    else
        div=bsindx-1;
    end
    
    BETA=sum(Bs(1:div).*Sx(1:div))/sum(Sx(1:div))+(0.1*randn);
    BETA=min(MAX_BETA,max(MIN_BETA,BETA));
    % use roullette wheel selection (without ranking) when females choose a
    % mat  e
    
    %%
    vindx=mindx;
    [vF idx]=sort(FF(vindx),'descend');
    vindx=vindx(idx);
    rM=(1:length(vindx))/sum(1:length(vindx));
    rMc=cumsum(rM);
    %%
    %     rM=FF(mindx)/sum(FF(mindx));
    %
    %     rMc=cumsum((1-rM)./sum(1-rM));
    for f=findx
        % if the female is carrying an egg then let it lay the egg with
        % probability EGG_P
        
        % if the female is carrying an egg then it moves greedily through
        % random mutation in a dimension
        % the amount of mutation depends upon BETA and the range of
        % variables in that dimension
        
        if(haseggs(f))
            dm=ceil(rand*nv);
            xtemp=P(:,f);
            %xtemp(dm)=(Rv(dm,1)+Rv(dm,2))/2+BETA*(rand(size(dm))-0.5).*(Rv(dm,2)-Rv(dm,1));
            xtemp(dm)=xtemp(dm)+(BETA)*(randn(size(dm)));
            % xtemp=P(:,f)+BETA.*(P(:,ceil(rand*MAX_ADULTS))-P(:,ceil(rand*MAX_ADULTS)));
            [xf_fit NC]=evalfcn(fcn_eval,xtemp,NC,fcn_param);
            if(xf_fit<FF(f)) % greedy search
                P(:,f)=xtemp;
                FF(f)=xf_fit;
            end
            if(rand<EGG_P)
                E(:,end+1)=P(:,f);
                EF(end+1)=FF(f);
                haseggs(f)=haseggs(f)-1;
            end
        else %otherwise
            % OTHERWISE the female either performs a random mutation or a
            % cross over depending upon the value of mutation probability PM
            % In crossover the female selects a male (using roulette wheel
            % selection) and crosses over some variables with it (i.e.
            % moves to a location near it)
            % in mutation the female moves randomly around in one dimension
            if(rand>PC)
                x1=ceil(rand*MAX_ADULTS);x2=ceil(rand*MAX_ADULTS);
                if(FF(x1)>FF(x2))
                    xxx=x1;
                    x1=x2;
                    x2=xxx;
                end
                [zzz idxx]=min(FF);
                xtemp=gg*P(:,idxx)+(1-gg)*P(:,ceil(rand*MAX_ADULTS))+BETA.*(P(:,x1)-P(:,x2));
                [xf_fit NC]=evalfcn(fcn_eval,xtemp,NC,fcn_param);
                if(xf_fit<FF(f))
                    if(bsindx<=H)
                        Bs(bsindx)=BETA;
                        Sx(bsindx)=(FF(f)-xf_fit);
                    else
                        Bs=[Bs(2:end) BETA];
                        Sx=[Sx(2:end) (FF(f)-xf_fit)];
                    end
                    bsindx=bsindx+1;
                    FF(f)=xf_fit;
                    P(:,f)=xtemp;
                    %else
                    
                end
            else
                if(~isnan(rMc))
                    mm=vindx(min(find(rand<rMc)));
                elseif ~isempty(mindx)
                    mm=vindx(1);
                else
                    mm=ceil(rand*length(P));
                end
                M=P(:,mm);
                if(~isempty(M))
                    xtemp=xover_xchg(M,P(:,f),PR);
                    [xf_fit NC]=evalfcn(fcn_eval,xtemp,NC,fcn_param);
                    if(xf_fit<FF(f))
                        P(:,f)=xtemp;
                        FF(f)=xf_fit;
                        haseggs(f)=ceil(rand*N_EGGS);
                    end
                else
                    % no males!!
                end
            end
        end
    end
    vindx=find( G );
    [vF idx]=sort(FF(vindx),'descend');
    vindx=vindx(idx);
    rM=(1:length(vindx))/sum(1:length(vindx));
    rMc=cumsum(rM);
    
    for m=mindx
        % if(rand<PM)
        %P(:,m)=P(:,m)+BETA.*(P(:,ceil(rand*MAX_ADULTS))-P(:,ceil(rand*MAX_ADULTS)));
        %[xf_fit NC]=evalfcn(xtemp,NC);
        %sP(:,m)=xtemp;
        if(rand<PC)%CHANGED ***************************************
            F=P(:,vindx(min(find(rand<rMc))));
            if(~isempty(F))
                P(:,m)=P(:,m)+rand*(F-P(:,m)); %HOW MUCH TO MOVE TOWARDS THE SELECTED FEMALE
            end
        else
            dm=randperm(nv);dm=dm(1:ceil(PM*rand*nv));%ceil(rand*nv);
            % dm=1:nv;
            dm=dm(:);
            P(dm,m)=P(dm,m)+(BETA)*(randn(size(dm)));%.*(Rv(dm,2)-Rv(dm,1));%Rv(dm,1)+2*(rand(size(dm))-0.5).*(Rv(dm,2)-Rv(dm,1));
            %
            
        end
        [FF(m) NC]=evalfcn(fcn_eval,P(:,m),NC,fcn_param);
        %end
    end
    % for the male, select a female using rank based routllette wheel
    % selection as males have only the objective of mating more often
    %restore elite
    if(FF(indxB)>evalfcn(fcn_eval,B,NC,fcn_param))
        P(:,indxB)=B;
        FF(indxB)=vv;
    end
    NC=NC+1;
    %bsindx
    %  MAX_EGGS=ceil(nnz(EF<mean(FF))/2);
    if(size(E,2)>MAX_EGGS)
        [EF indx]=sort(EF,'ascend');
        indx=indx(1:MAX_EGGS);
        E=E(:,indx);
        EF=EF(:,indx);
    end
    %     MAX_EGGS
    %     size(EF)
    %     size(E)
    if((vv<=FMIN)|| (NC>MAX_NC)|| (t>=MAX_T))
        state.FF=FF;
        state.EF=EF;
        state.P=P;
        state.E=E;
        state.Bs=Bs;
        state.Sx=Sx;
        state.BETA=BETA;
        state.bsindx=bsindx;
        state.haseggs=haseggs;
        state.G=G;
        state.xxdata=xxdata;
        state.NC=NC;
        state.vv=vv;
        state.B=B;
        state.indxB=indxB;
        break;
    end
end
if(vv>FMIN)
    S=0;
else
    S=1;
end


function [f NC]=evalfcn(h,x,NC,fcn_param)

NC=NC+1;
if(isempty(fcn_param))
    f=feval(h,x');
else
    f=feval(h,x',fcn_param);
end

function c=xover_xchg(M,F,p)
r=rand(length(F),1)<p;
c=F;
c(r)=M(r);

